#Check if cython code has been compiled
import os
import subprocess
print("Importing AfterImage Cython Library")
if not os.path.isfile("AfterImage.c"): #has not yet been compiled, so try to do so...
    cmd = "python setup.py build_ext --inplace"
    subprocess.call(cmd,shell=True)
#Import dependencies
import netStat as ns
import csv
import numpy as np
print("Importing Scapy Library")
from scapy.all import *
import os.path
import platform
import subprocess


#Extracts Kitsune features from given pcap file one packet at a time using "get_next_vector()"
# If wireshark is installed (tshark) it is used to parse (it's faster), otherwise, scapy is used (much slower).
# If wireshark is used then a tsv file (parsed version of the pcap) will be made -which you can use as your input next time
class FE:
    def __init__(self,file_path,limit=np.inf):
        self.path = file_path
        self.limit = limit
        self.parse_type = None #unknown
        self.curPacketIndx = 0
        self.scapyin = None #used for parsing pcap with scapy
        self.triggerMAC1 = None
        self.triggerMAC2 = None

        ### Prep pcap ##
        self.__prep__()

        ### Prep Feature extractor (AfterImage) ###
        maxHost = 100000000000
        maxSess = 100000000000
        self.nstat = ns.netStat(np.nan, maxHost, maxSess)


    def __prep__(self):
        ### Find file: ###
        if not os.path.isfile(self.path):  # file does not exist
            print("File: " + self.path + " does not exist")
            raise Exception()

        ### check file type ###
        type = self.path.split('.')[-1]

        ##If file is pcap
        if type == "pcap" or type == 'pcapng':
            self.parse_type = "scapy"
        else:
            print("File: " + self.path + " is not a tsv or pcap file")
            raise Exception()

        print("Reading PCAP file via Scapy...")
        try:
            self.scapyin = rdpcap(self.path)
            self.limit = len(self.scapyin)
            print("Loaded " + str(len(self.scapyin)) + " Packets.")
        except:
            print("error...restarting prep")
            time.sleep(10)
            self.__prep__()

    def get_next_vector(self):
         if self.curPacketIndx == self.limit:
            if self.parse_type == 'tsv':
                self.tsvinf.close()
            return []

         ### Parse next packet ###
         if self.parse_type == "tsv":
             row = self.tsvin.__next__()
             IPtype = np.nan
             timestamp = row[0]
             framelen = row[1]
             srcIP = ''
             dstIP = ''
             if row[4] != '':  # IPv4
                 srcIP = row[4]
                 dstIP = row[5]
                 IPtype = 0
             elif row[17] != '':  # ipv6
                 srcIP = row[17]
                 dstIP = row[18]
                 IPtype = 1
             srcproto = row[6] + row[8]  # UDP or TCP port: the concatenation of the two port strings will will results in an OR "[tcp|udp]"
             dstproto = row[7] + row[9]  # UDP or TCP port
             srcMAC = row[2]
             dstMAC = row[3]
             if srcproto == '':  # it's a L2/L1 level protocol
                 if row[12] != '':  # is ARP
                     srcproto = 'arp'
                     dstproto = 'arp'
                     srcIP = row[14]  # src IP (ARP)
                     dstIP = row[16]  # dst IP (ARP)
                     IPtype = 0
                 elif row[10] != '':  # is ICMP
                     srcproto = 'icmp'
                     dstproto = 'icmp'
                     IPtype = 0
                 elif srcIP + srcproto + dstIP + dstproto == '':  # some other protocol
                     srcIP = row[2]  # src MAC
                     dstIP = row[3]  # dst MAC

         elif self.parse_type == "scapy":
             packet = self.scapyin[self.curPacketIndx]
             IPtype = np.nan
             timestamp = packet.time
             framelen = len(packet)
             if(packet is None):
                 print("\nPacket is none!\n")
                 return []
             srcMAC = None
             dstMAC = None

             if packet.haslayer(IP):  # IPv4
                 srcIP = packet[IP].src
                 dstIP = packet[IP].dst
                 srcMAC = packet.src
                 dstMAC = packet.dst
                 IPtype = 0
             elif packet.haslayer(IPv6):  # ipv6
                 srcIP = packet[IPv6].src
                 dstIP = packet[IPv6].dst
                 srcMAC = packet.src
                 dstMAC = packet.dst
                 IPtype = 1
             else:
                 srcIP = ''
                 dstIP = ''

             if packet.haslayer(TCP):
                 srcproto = str(packet[TCP].sport)
                 dstproto = str(packet[TCP].dport)
             elif packet.haslayer(UDP):
                 srcproto = str(packet[UDP].sport)
                 dstproto = str(packet[UDP].dport)
             else:
                 srcproto = ''
                 dstproto = ''

             srcMAC = packet.src
             dstMAC = packet.dst
             self.triggerMAC1 = srcMAC
             self.triggerMAC2 = dstMAC
             if srcproto == '':  # it's a L2/L1 level protocol
                 if packet.haslayer(ARP):  # is ARP
                     srcproto = 'arp'
                     dstproto = 'arp'
                     srcIP = packet[ARP].psrc  # src IP (ARP)
                     dstIP = packet[ARP].pdst  # dst IP (ARP)
                     IPtype = 0
                     #print("ARP MESSAGE: " + str(srcMAC) + "  " + str(dstMAC) + "\n\n")
                 elif packet.haslayer(ICMP):  # is ICMP
                     srcproto = 'icmp'
                     dstproto = 'icmp'
                     IPtype = 0
                     #print("ICMP MESSAGE\n\n")
                 elif srcIP + srcproto + dstIP + dstproto == '':  # some other protocol
                     srcIP = packet.src  # src MAC
                     dstIP = packet.dst  # dst MAC
                     #print("UNKNOWN PROTOCOL\n\n")

         else:
             return []

         self.curPacketIndx = self.curPacketIndx + 1


         ### Extract Features
         try:
             return self.nstat.updateGetStats(IPtype, srcMAC, dstMAC, srcIP, srcproto, dstIP, dstproto,
                                                 int(framelen),
                                                 float(timestamp))
         except Exception as e:
             print(e)
             print("\n\nException in extracting features!")
             print("IPtype: " + str(IPtype) + " srcMAC: " + str(srcMAC) + " dstMAC: " + str(dstMAC) + " srcIP: " + str(srcIP) + " dstIP: " + str(dstIP) + "\n\n")
             return []

    def get_num_features(self):
        return len(self.nstat.getNetStatHeaders())

    def updateFile(self):
        curPath = os.getcwd()
        dataPath = "/live_data"
        if(dataPath not in curPath):
            dataPath = curPath + "/live_data"
        else:
            dataPath = curPath

        os.chdir(dataPath)
        files = sorted(os.listdir(os.getcwd()), key=os.path.getmtime)
        file_count = len(files)
        if("/" in self.path):
            curFile = self.path.rsplit('/',1)[1]
        else:
            curFile = self.path
        index = files.index(curFile)

        if(index + 1 < file_count):
            self.path = dataPath + "/" + str(files[index + 1])
            self.curPacketIndx = 0
            os.chdir(curPath)
            self.__prep__()
            return 1
        else:
            print("No more files to process!")
            os.chdir(curPath)
            return -1

    def getMAC(self):
        return [self.triggerMAC1, self.triggerMAC2]
