from subprocess import call
from cmd import Cmd
from os import isatty
from select import poll, POLLIN
import select
import errno
import sys
import time
import os, os.path
import atexit
import time
import re

from Kitsune import Kitsune
import numpy as np

class CLI(Cmd):
    prompt = 'GDI> '
    intro = "\n\nGDI Security Smart Home Platform"

    def do_exit(self, inp):
        print("Exiting CLI...")
        return True

    def do_runAll(self, inp):
        print("Automated run on full system")
        # Start fingerprinting services
        # Start IDS in live mode

    def do_runIDS(self, inp):
        if(inp != ""):
            print("Input file present: " + inp)
            # Run Kitsune statically on the path and exit after reading entire file
            packet_limit = np.Inf #the number of packets to process

            # KitNET params:
            maxAE = 10 #maximum size for any autoencoder in the ensemble layer
            FMgrace = 5000 #the number of instances taken to learn the feature mapping (the ensemble's architecture)
            ADgrace = 50000 #the number of instances used to train the anomaly detector (ensemble itself)
            cutoff = 10 #the value of RMSE to trigger IDS on

            # Build Kitsune
            K = Kitsune(inp,packet_limit,maxAE,FMgrace,ADgrace,cutoff)

            print("Running Kitsune:")
            RMSEs = []
            i = 0
            start = time.time()
            # Here we process (train/execute) each individual packet.
            # In this way, each observation is discarded after performing process() method.
            while True:
                i+=1
                if i % 1000 == 0:
                    print(i)

                rmse = K.proc_next_packet()
                if rmse == -1:
                    break   # This is where we used to wait for more files

                elif(rmse is not None and rmse != -1):
                    #if(rmse >= cutoff):
                        #print("TRIGGERED MACS:")
                        #print(K.triggerMACs)

                    RMSEs.append(rmse)

            stop = time.time()
            print("Complete. Time elapsed: "+ str(stop - start))

            # Plot results
            filteredRMSEs = list(filter(None, RMSEs))
            benignSample = np.log(filteredRMSEs[FMgrace+ADgrace+1:100000])
            logProbs = norm.logsf(np.log(filteredRMSEs), np.mean(benignSample), np.std(benignSample))
            print("Plotting results")
            from matplotlib import pyplot as plt
            from matplotlib import cm
            plt.figure(figsize=(10,5))
            fig = plt.scatter(range(FMgrace+ADgrace+1,len(filteredRMSEs)),filteredRMSEs[FMgrace+ADgrace+1:],s=0.1,c=logProbs[FMgrace+ADgrace+1:],cmap='RdYlGn')
            plt.title("Anomaly Scores from Kitsune's Execution Phase")
            plt.ylabel("RMSE")
            plt.xlabel("Time elapsed [min]")
            plt.show()

        else:
            print("Running IDS in live time")
            # Create dir if doesn't exists, clear it if it does
            dataPath = "/home/roger/Kitsune-py/live_data"
            tsvPath = dataPath + "/tsv"
            logPath = dataPath + "/logs"
            curDir = os.getcwd()
            if(os.path.isdir(dataPath)):
                print("live_data already present")
                for a_file in os.listdir(dataPath):
                    file_path = os.path.join(dataPath, a_file)
                    try:
                        if(os.path.isfile(file_path)):
                            os.unlink(file_path)
                    except Exception as e:
                        print(e)

                if(os.path.isdir(tsvPath)):
                    print("tsv already present")
                    for a_file in os.listdir(tsvPath):
                        file_path = os.path.join(tsvPath, a_file)
                        try:
                            if(os.path.isfile(file_path)):
                                os.unlink(file_path)
                        except Exception as e:
                            print(e)

                if(os.path.isdir(logPath)):
                    print("log already present")
                    for a_file in os.listdir(logPath):
                        file_path = os.path.join(logPath, a_file)
                        try:
                            if(os.path.isfile(file_path)):
                                os.unlink(file_path)
                        except Exception as e:
                            print(e)

            else:
                print("created live_data")
                os.mkdir(dataPath)
                os.chdir(dataPath)
                os.mkdir("tsv")
                os.mkdir("log")
                os.chdir(curDir)

            # Switch to directory and capture training data
            os.chdir(dataPath)
            p = subprocess.Popen("sudo tcpdump -i any -w training69.pcap -c 50", stdout=subprocess.PIPE, shell=True)
            p.wait()
            if(p.returncode == 0):
                print("Captured training data successfully!")
                self.preprocess()
            else:
                print("fug, training data failed...aborting")
                return False

            # After successful training capture in 10 sec intervals

            # Start Kitsune

            # run mac_lookup.py <mac_address> in finger folder with anomaly MAC

    def preprocess(self, file_in=None):
        if(file_in is not None):
            find = re.compile(r"^[^.]*")
            file_out = re.search(find, file_in).group(0) + ".tsv"
            cmd = ("tshark -r " + file_in + " -T fields -e frame.time_epoch -e frame.len -e eth.src "
                    "-e eth.dst -e ip.src -e ip.dst -e tcp.srcport -e tcp.dstport -e udp.srcport -e udp.dstport "
                    "-e icmp.type -e icmp.code -e arp.opcode -e arp.src.hw_mac -e arp.src.proto_ipv4 "
                    "-e arp.dst.hw_mac -e arp.dst.proto_ipv4 -e ipv6.src -e ipv6.dst -E header=y "
                    "-E occurrence=f > tsv/" + file_out)
            call(cmd, shell=True)
        else:
            # Check what directory we're in
            changeBack = False
            curDir = os.getcwd()
            if(os.getcwd().split("/")[-1] != "live_data"):
                os.chdir(dataPath)
                changeBack = True

            # Find oldest, non-tsv file and process it
            files = sorted(os.listdir(os.getcwd()), key=os.path.getmtime)
            for f in files:
                if(len(f.split(".")) >= 2):
                    if(f.split(".")[1] == "pcap" or f.split(".")[1] == "pcapng"):
                        file_out = str(f.split(".")[0]) + ".tsv"
                        cmd = ("tshark -r " + f + " -T fields -e frame.time_epoch -e frame.len -e eth.src "
                                "-e eth.dst -e ip.src -e ip.dst -e tcp.srcport -e tcp.dstport -e udp.srcport -e udp.dstport "
                                "-e icmp.type -e icmp.code -e arp.opcode -e arp.src.hw_mac -e arp.src.proto_ipv4 "
                                "-e arp.dst.hw_mac -e arp.dst.proto_ipv4 -e ipv6.src -e ipv6.dst -E header=y "
                                "-E occurrence=f > tsv/" + file_out)
                        call(cmd, shell=True)

    def do_runFinger(self, inp):
        print("Starting fingerprinting services")
        cmd = "sudo systemctl start mqp_finger.service"
        call(cmd, shell=True)

    def do_checkFinger(self, inp):
        print("Checking status of fingerprinting services")
        cmd = "sudo systemctl status mqp_finger.service"
        call(cmd, shell=True)

    def do_stopFinger(self, inp):
        print("Stoping fingerprinting services")
        cmd = "sudo systemctl stop mqp_finger.service"
        call(cmd, shell=True)

    def help_runFinger(self):
        print("Starts fingerprinting services to identify devices on network and check \nfor new devices")

    def help_checkFinger(self):
        print("Checks the status of the fingerprinting service")

    def help_stopFinger(self):
        print("Stops the fingerprinting service")

    def help_runIDS(self):
        print("Runs the IDS in real time if no input file is given. If a file is given it \nwill run Kitsune on the prerecorded pcap")

    def help_runAll(self):
        print("Runs the GDI security suite and all associated modules")

    def help_exit(self):
        print("Exits CLI")

CLI().cmdloop()
