from scapy.all import sniff, DHCP, Ether
from requests import get
from json import loads, dump
from os.path import isfile, join
from sys import argv

JSON_DUMP_PATH = argv[1]
KEY = '8b337919121d3c0a825b9c90ca21de737cce4f09' # Dillon

def extract_dhcp_fingerprint(packet):
    if DHCP not in packet: return None
    options = packet[DHCP].options
    tuples = [item for item in options if item.__class__.__name__ == 'tuple']
    mapping = {key: value for (key, value) in tuples}
    if 'param_req_list' not in mapping: return False
    else: return mapping['param_req_list']

def extract_mac_address(packet):
    if Ether not in packet: return False
    return packet[Ether].src

def query_api(dhcp, mac):
    url = 'https://api.fingerbank.org/api/v2/combinations/interrogate?key='
    headers = {'Content-type': 'application/json'}
    data = '{"dhcp_fingerprint": "%s", "mac": "%s"}' % (dhcp, mac)
    response = get('%s%s' % (url, KEY), headers=headers, data=data)
    return loads(response.text)

def main(packet):
    dhcp_array = extract_dhcp_fingerprint(packet)
    mac = extract_mac_address(packet)
    if not dhcp_array: return
    if not mac: return
    dhcp_string = ''.join('%s,' % str(integer) for integer in dhcp_array)[:-1]
    print('MAC address %s and DHCP fingerprint %s recovered' % (mac, dhcp_string))
    path = join(JSON_DUMP_PATH, "%s.fingerbank.json" % mac.lower().replace(":", "."))
    if isfile(path): print('JSON from Fingerbank API is already on file for this device')
    else:
        print('This is an unknown device, querying Fingerbank API for JSON')
        json = query_api(dhcp_string, mac)
        print('Writing JSON to record')
        with open(path, 'w') as target:
            dump(json, target)
            print('Success')

f = 'udp and (port 67 or port 68)'
i = 'wlan0'
print('Filtering [ %s ] on %s...' % (f, i))
sniff(iface=i, filter=f, prn=main)

