from subprocess import check_output
from re import findall
from scapy.all import ARP, Ether, srp
from sys import argv
from os.path import isfile, join
from json import dump

JSON_DUMP_PATH = argv[1]
TRAIL = "[^:]*: ([^\n]+)\n"

def map_addresses(ip_range):
    #command = "nmap -sP %s" % ip_range
    output = check_output(['nmap', '-sP', ip_range])
    ip_addresses = findall("((?:[0-9]{1,3}[.]){3}[0-9]{1,3})", output)
    mac_addresses = findall("((?:[0-9A-Fa-f]{2}[:]){5}[0-9A-Fa-f]{2})", output)
    mapping = dict(zip(ip_addresses, mac_addresses))
    return mapping

def nmap(ip_address):
    #command = 'nmap -O -Pn --host-timeout 3m %s' % ip_address
    return check_output(['nmap', '-O', '-Pn', '--host-timeout', '3m', ip_address])
    #return check_output(command.split(' '))

def find_mac_address(nmap_output):
    return findall("MAC Address" + TRAIL, nmap_output)

def find_device_type(nmap_output):
    return findall("Device type" + TRAIL, nmap_output)

def find_running(nmap_output):
    return findall("Running" + TRAIL, nmap_output) 

def find_os_cpe(nmap_output):
    return findall("OS CPE" + TRAIL, nmap_output)

def find_os_details(nmap_output):
    return findall("OS details" + TRAIL, nmap_output)

def find_aggressive_os_guesses(nmap_output):
    return findall("Aggressive OS guesses" + TRAIL, nmap_output)

def map_nmap_output(nmap_output):
    result = {}
    result['mac_address'] = find_mac_address(nmap_output)
    result['device_type'] = find_device_type(nmap_output)
    result['running'] = find_running(nmap_output)
    result['os_cpe'] = find_os_cpe(nmap_output)
    result['os_details'] = find_os_details(nmap_output)
    result['aggressive_os_guesses'] = find_aggressive_os_guesses(nmap_output)
    return result

def main():
    print("Mapping IP addresses to MAC addresses")
    addresses = map_addresses("192.168.4.0/24")
    print("%s devices were discovered" % len(addresses))
    for ip, mac in addresses.items():
        print("%s at %s" % (mac.lower().replace(':', '.'), ip))
    for ip, mac in addresses.items():
        path = join(JSON_DUMP_PATH, "%s.nmap.json" % mac.lower().replace(":", "."))
        if isfile(path):
            print("Discovered known device (MAC %s) at IP %s" % (mac, ip))
        if not isfile(path):
            print("Discovered unknown device (MAC %s) at IP %s" % (mac, ip))
            print("Running nmap OS detection against this device")
            nmap_output = nmap(ip)
            if "host timeout\n" in nmap_output:
                print('The host at %s timed out, withholding JSON dump' % ip)
                continue
            print('Mapping nmap output to JSON and dumping')
            nmap_mapping = map_nmap_output(nmap_output)
            with open(path, 'w') as target:
                dump(nmap_mapping, target)

main()

